// defines classes and declarations for SEGDWT
#ifndef SEGDWT_H
#define SEGDWT_H

#include <iostream>
#include <vector>
#include <list>

#define DEBUG debug_

// basic processing type declaration
typedef double type;

// filetype for vectors
typedef std::vector<type> vecType;
typedef vecType::iterator vecIter;

// std::cout overloads
using std::ostream;
ostream& operator<<(ostream& stream, vecType& vec);


class SegDWT {
	// class declarations
	// segment is used for coefs storage
	struct Segment {
		struct WaveletCoefs {
			vecType a;
			vecType d;
		};
		bool					 isLast;		// if
		std::list<WaveletCoefs>	 coefs;			// contains result coefs. Scheme: {A D} {A D} {A D} ... 
		Segment(unsigned depth, bool lst);
	};	
	
	enum workType {dyaddown, leftcut, rightcut};
	
	// hard params (constructor-based)
	unsigned segSize_;
	unsigned wDepth_;
	unsigned twoToJ_;
	unsigned extLen_;
	
	bool debug_;
	
	// weak params
	unsigned segCount_;		// count of segments, init after file load
	unsigned lastSegSize_;	// size of last segment
	unsigned n_;			// algorithm step
	
	// storage
	vecType LoDFilter_;
	vecType LoRFilter_;
	vecType HiDFilter_;		// filters
	vecType HiRFilter_;
	
	vecType signal_;		// input signal
	
	std::list<Segment>	seg_;	// resulting segments
	
	// private methods
	int getLeftExtSize(int segment);
	int getRightExtSize(int segment);
	void extendLeft(int segment, vecType& vector);
	void extendRight(int segment, int specR, vecType& vector);
	void mlwd(SegDWT::Segment &seg, vecType &src);
	void conv(vecType& signal, vecType& filter, vecType& result);
	void changeSignal(vecType& signal, workType type, unsigned cutOff=0);
		
public:
	// exceptions
	class ExcFilterFile {};
	class ExcSignalFile {};
	class ExcSaveFile {};
	
	// PUBLIC INTERFACE
	// constructor
	SegDWT(unsigned segmentSize, unsigned depth, bool debg);
	// load signal
	void loadSignal(const char *filename);
	// load filter
	void loadFilter(const char *filename);
	// process
	void performTransform();
	// save signal
	void saveResult(const char *filename);
};

#endif